/******************************************************************************
 * @file     descriptors.c
 * @brief    NUC122 series USBD descriptor file
 *
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
/*!<Includes */
#include "NUC122.h"
#include "usbd_user.h"
#include "hid.h"

/*!<USB HID Report Descriptor */
uint8_t HID_MouseReportDescriptor[] =
{
    0x05, 0x01,     /* Usage Page(Generic Desktop Controls) */
    0x09, 0x02,     /* Usage(Mouse) */
    0xA1, 0x01,     /* Collection(Application) */
    0x09, 0x01,     /* Usage(Pointer) */
    0xA1, 0x00,     /* Collection(Physical) */
    0x05, 0x09,     /* Usage Page(Button) */
    0x19, 0x01,     /* Usage Minimum(0x1) */
    0x29, 0x03,     /* Usage Maximum(0x3) */
    0x15, 0x00,     /* Logical Minimum(0x0) */
    0x25, 0x01,     /* Logical Maximum(0x1) */
    0x95, 0x03,     /* Report Count(0x3) */
    0x75, 0x01,     /* Report Size(0x1) */
    0x81, 0x02,     /* Input(3 button bit) */
    0x95, 0x01,     /* Report Count(0x1) */
    0x75, 0x05,     /* Report Size(0x5) */
    0x81, 0x01,     /* Input(5 bit padding) */
    0x05, 0x01,     /* Usage Page(Generic Desktop Controls) */
    0x09, 0x30,     /* Usage(X) */
    0x09, 0x31,     /* Usage(Y) */
    0x15, 0x81,     /* Logical Minimum(0x81)(-127) */
    0x25, 0x7F,     /* Logical Maximum(0x7F)(127) */
    0x75, 0x08,     /* Report Size(0x8) */
    0x95, 0x02,     /* Report Count(0x2) */
    0x81, 0x06,     /* Input(1 byte) */
    0xC0,           /* End Collection */
    0xC0,           /* End Collection */
};

#define HID_MOUSE_REPORT_DESCRIPTOR_SIZE \
    sizeof (HID_MouseReportDescriptor) / sizeof(HID_MouseReportDescriptor[0])
const uint32_t g_HID_u32MouseReportDescriptorSize = HID_MOUSE_REPORT_DESCRIPTOR_SIZE;

uint8_t g_au8MouseReport[HID_REPORT_SIZE_MS];
const uint32_t g_u32MouseReportSize = sizeof(g_au8MouseReport) / sizeof(g_au8MouseReport[0]);


/*!<USB HID Report Descriptor */
uint8_t HID_KeyboardReportDescriptor[] =
{
    0x05, 0x01,     /* Usage Page(Generic Desktop Controls) */
    0x09, 0x06,     /* Usage(Keyboard) */
    0xA1, 0x01,     /* Collection(Application) */
    0x85, 0x01,     /* Report ID (1) */
    0x05, 0x07,     /* Usage Page(Keyboard/Keypad) */
    0x19, 0xE0,     /* Usage Minimum(0xE0) */
    0x29, 0xE7,     /* Usage Maximum(0xE7) */
    0x15, 0x00,     /* Logical Minimum(0x0) */
    0x25, 0x01,     /* Logical Maximum(0x1) */
    0x75, 0x01,     /* Report Size(0x1) */
    0x95, 0x08,     /* Report Count(0x8) */
    0x81, 0x02,     /* Input (Data) => Modifier byte */
    0x95, 0x01,     /* Report Count(0x1) */
    0x75, 0x08,     /* Report Size(0x8) */
    0x81, 0x01,     /* Input (Constant) => Reserved byte */
    0x95, 0x05,     /* Report Count(0x5) */
    0x75, 0x01,     /* Report Size(0x1) */
    0x05, 0x08,     /* Usage Page(LEDs) */
    0x19, 0x01,     /* Usage Minimum(0x1) */
    0x29, 0x05,     /* Usage Maximum(0x5) */
    0x91, 0x02,     /* Output (Data) => LED report */
    0x95, 0x01,     /* Report Count(0x1) */
    0x75, 0x03,     /* Report Size(0x3) */
    0x91, 0x01,     /* Output (Constant) => LED report padding */
    0x95, 0x06,     /* Report Count(0x6) */
    0x75, 0x08,     /* Report Size(0x8) */
    0x15, 0x00,     /* Logical Minimum(0x0) */
    0x25, 0x65,     /* Logical Maximum(0x65) */
    0x05, 0x07,     /* Usage Page(Keyboard/Keypad) */
    0x19, 0x00,     /* Usage Minimum(0x0) */
    0x29, 0x65,     /* Usage Maximum(0x65) */
    0x81, 0x00,     /* Input (Data) */
    0xC0,           /* End Collection */

    0x05, 0x0C,     /* Usage Page (Consumer Devices) */
    0x09, 0x01,     /* Usage (Consumer Control) */
    0xA1, 0x01,     /* Collection: (Application) */
    0x85, 0x02,     /* Report ID (2) */
    0x15, 0x00,     /* Logical Minimum */
    0x25, 0x01,     /* Logical Maximum */
    0x09, 0xE2,     /* Usage (Mute) */
    0x09, 0xE9,     /* Usage (Volume+) */
    0x09, 0xEA,     /* Usage (Volume-) */
    0x09, 0x6F,     /* Usage (Brightness+) */
    0x09, 0x70,     /* Usage (Brightness-) */
    0x75, 0x01,     /* Report Size (1) */
    0x95, 0x05,     /* Report Count (5) */
    0x81, 0x02,     /* Input: (Data, Variable, Absolute) */
    0x05, 0x01,     /* Usage Page (Generic Desktop Controls) */
    0x09, 0x82,     /* Usage (System Sleep) */
    0x75, 0x01,     /* Report Size (1) */
    0x95, 0x01,     /* Report Count (1) */
    0x81, 0x02,     /* Input: (Data, Variable, Absolute) */
    0x75, 0x01,     /* Report Size (1) */
    0x95, 0x02,     /* Report Count (2) */
    0x81, 0x03,     /* Input: (Constant, Variable, Absolute) */
    0xC0,           /* End Collection */
};

#define HID_KEYBOARD_REPORT_DESCRIPTOR_SIZE \
    sizeof(HID_KeyboardReportDescriptor) / sizeof(HID_KeyboardReportDescriptor[0])
const uint32_t g_HID_u32KeyboardReportDescriptorSize = HID_KEYBOARD_REPORT_DESCRIPTOR_SIZE;

uint8_t g_au8KeyboardReport[HID_REPORT_SIZE_KB];
const uint32_t g_u32KeyboardReportSize = sizeof(g_au8KeyboardReport) / sizeof(g_au8KeyboardReport[0]);

/*----------------------------------------------------------------------------*/
/*!<USB Device Descriptor */
uint8_t gu8DeviceDescriptor[] =
{
    LEN_DEVICE,               /* bLength */
    DESC_DEVICE,              /* bDescriptorType */
    0x10, 0x01,               /* bcdUSB */
    0x00,                     /* bDeviceClass */
    0x00,                     /* bDeviceSubClass */
    0x00,                     /* bDeviceProtocol */
    EP0_MAX_PKT_SIZE,         /* bMaxPacketSize0 */
    USBD_VID & 0x00FF,        /* idVendor */
    (USBD_VID & 0xFF00) >> 8,
    USBD_PID & 0x00FF,        /* idProduct */
    (USBD_PID & 0xFF00) >> 8,
    0x00, 0x00,               /* bcdDevice */
    0x01,                     /* iManufacture */
    0x02,                     /* iProduct */
    0x00,                     /* iSerialNumber - no serial */
    0x01                      /* bNumConfigurations */
};

/*!<USB Configure Descriptor */
uint8_t gu8ConfigDescriptor[] =
{
    LEN_CONFIG,               /* bLength */
    DESC_CONFIG,              /* bDescriptorType */
    LEN_CONFIG_AND_SUBORDINATE & 0x00FF,   /* wTotalLength */
    (LEN_CONFIG_AND_SUBORDINATE & 0xFF00) >> 8,
    0x02,                     /* bNumInterfaces */
    0x01,                     /* bConfigurationValue */
    0x00,                     /* iConfiguration */
    0x80 | (USBD_SELF_POWERED << 6) | (USBD_REMOTE_WAKEUP << 5),  /* bmAttributes */
    USBD_MAX_POWER,           /* MaxPower */

    /* HID Interface Descriptor*/
    LEN_INTERFACE,  /* bLength */
    DESC_INTERFACE, /* bDescriptorType */
    0x00,           /* bInterfaceNumber */
    0x00,           /* bAlternateSetting */
    0x01,           /* bNumEndpoints */
    0x03,           /* bInterfaceClass */
    0x01,           /* bInterfaceSubClass */
    HID_MOUSE,      /* bInterfaceProtocol */
    0x00,           /* iInterface */

    /* HID Descriptor */
    LEN_HID,        /* bLength */
    DESC_HID,       /* bDescriptorType */
    0x10, 0x01,     /* bcdHID */
    0x00,           /* bCountryCode */
    0x01,           /* bNumDescriptors */
    DESC_HID_RPT,   /* bDescriptorType */
    HID_MOUSE_REPORT_DESCRIPTOR_SIZE & 0x00FF,  /* wDescriptorLength */
    (HID_MOUSE_REPORT_DESCRIPTOR_SIZE & 0xFF00) >> 8,

    /* EP Descriptor: interrupt in. */
    LEN_ENDPOINT,                    /* bLength */
    DESC_ENDPOINT,                   /* bDescriptorType */
    (INT_IN_EP_NUM_MS | EP_INPUT),   /* bEndpointAddress */
    EP_INT,                          /* bmAttributes */
    EP2_MAX_PKT_SIZE & 0x00FF,       /* wMaxPacketSize */
    (EP2_MAX_PKT_SIZE & 0xFF00) >> 8,
    HID_DEFAULT_INT_IN_INTERVAL,     /* bInterval */

    /* HID Interface Descriptor*/
    LEN_INTERFACE,  /* bLength */
    DESC_INTERFACE, /* bDescriptorType */
    0x01,           /* bInterfaceNumber */
    0x00,           /* bAlternateSetting */
    0x01,           /* bNumEndpoints */
    0x03,           /* bInterfaceClass */
    0x01,           /* bInterfaceSubClass */
    HID_KEYBOARD,   /* bInterfaceProtocol */
    0x00,           /* iInterface */

    /* HID Descriptor */
    LEN_HID,        /* bLength */
    DESC_HID,       /* bDescriptorType */
    0x10, 0x01,     /* bcdHID */
    0x00,           /* bCountryCode */
    0x01,           /* bNumDescriptors */
    DESC_HID_RPT,   /* bDescriptorType */
    HID_KEYBOARD_REPORT_DESCRIPTOR_SIZE & 0x00FF, /* wDescriptorLength */
    (HID_KEYBOARD_REPORT_DESCRIPTOR_SIZE & 0xFF00) >> 8,

    /* EP Descriptor: interrupt in. */
    LEN_ENDPOINT,   /* bLength */
    DESC_ENDPOINT,  /* bDescriptorType */
    (INT_IN_EP_NUM_KB | EP_INPUT), /* bEndpointAddress */
    EP_INT,         /* bmAttributes */
    /* wMaxPacketSize */
    EP3_MAX_PKT_SIZE & 0x00FF,
    (EP3_MAX_PKT_SIZE & 0xFF00) >> 8,
    HID_DEFAULT_INT_IN_INTERVAL,     /* bInterval */
};

/*!<USB Language String Descriptor */
uint8_t gu8StringLang[4] =
{
    4,              /* bLength */
    DESC_STRING,    /* bDescriptorType */
    0x09, 0x04
};

/*!<USB Vendor String Descriptor */
uint8_t gu8VendorStringDesc[] =
{
    16,             /* bLength */
    DESC_STRING,    /* bDescriptorType */
    'N', 0, 'u', 0, 'v', 0, 'o', 0, 't', 0, 'o', 0, 'n', 0
};

/*!<USB Product String Descriptor */
uint8_t gu8ProductStringDesc[] =
{
    22,             /* bLength */
    DESC_STRING,    /* bDescriptorType */
    'H', 0, 'I', 0, 'D', 0, ' ', 0, 'D', 0, 'e', 0, 'v', 0, 'i', 0, 'c', 0, 'e', 0
};


const uint8_t gu8StringSerial[26] =
{
    26,             /* bLength */
    DESC_STRING,    /* bDescriptorType */
    'B', 0, '0', 0, '2', 0, '0', 0, '0', 0, '6', 0, '0', 0, '9', 0, '2', 0, '1', 0, '1', 0, '4', 0
};


const uint8_t *gpu8UsbString[4] =
{
    gu8StringLang,
    gu8VendorStringDesc,
    gu8ProductStringDesc,
    gu8StringSerial
};

const S_USBD_USER_INFO_T gsUserInfo =
{
    gu8DeviceDescriptor,
    gu8ConfigDescriptor,
    gpu8UsbString,
    HID_KeyboardReportDescriptor,
    HID_MouseReportDescriptor
};
